from typing_extensions import Final  # noqa: F401

CLIENT_DELETED = 'client_deleted'  # type: Final
CLIENT_FETCHED = 'client_fetched'  # type: Final
CLIENTS_FETCHED = 'clients_fetched'  # type: Final
CLIENT_CREATED = 'client_created'  # type: Final
NEW_CLIENT_REGISTERED = 'new_client_registered'  # type: Final
UNREGISTERED_CLIENT = 'unregistered_client'  # type: Final
UNKNOWN_USER = 'unknown_user'  # type: Final
BAD_PASSWORD = 'bad_password'  # type: Final  # nosec
MISSING_SCOPES = 'missing_scopes'  # type: Final
UNKNOWN_COMPRESSION_FORMAT = 'unknown_compression_format'  # type: Final
EMAILS_DELIVERED_TO_CLIENT = 'emails_delivered_to_client'  # type: Final
EMAILS_FORMATTED_FOR_CLIENT = 'emails_formatted_for_client'  # type: Final
EMAILS_RECEIVED_FROM_CLIENT = 'emails_received_from_client'  # type: Final
EMAIL_RECEIVED_FOR_CLIENT = 'email_received_for_client'  # type: Final
EMAIL_DELIVERED_FROM_CLIENT = 'email_delivered_from_client'  # type: Final
EMAIL_STORED_FOR_CLIENT = 'email_stored_for_client'  # type: Final
EMAIL_STORED_FROM_CLIENT = 'email_stored_from_client'  # type: Final
USER_STORED_FROM_CLIENT = 'user_stored_from_client'  # type: Final
MAILBOX_EMAIL_INDEXED = 'mailbox_email_indexed'  # type: Final
