{% extends '_base.html' %}

{% from 'macros/form.html' import render_field %}
{% from 'macros/form.html' import render_button %}

{% block content %}{{ super() }}
<h1>{{ _('Change password') }}</h1>
<form action="{{ url_for_security('change_password') }}" method="post" name="change_password_form">
  {{ change_password_form.hidden_tag() }}

  {{ render_field(change_password_form.password, _('Enter your current password')) }}
  {{ render_field(change_password_form.new_password, _('Enter new password')) }}
  {{ render_field(change_password_form.new_password_confirm, _('Confirm new password')) }}
  {{ render_button(change_password_form.submit, _('Submit')) }}
</form>
{% endblock %}
